'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectQuery = exports.localEntities = exports.denormalize = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reselect = require('reselect');

var _requestStatus = require('./constants/requestStatus');

var _helpers = require('./helpers');

var denormalize = exports.denormalize = function denormalize(entities, id) {
  var memoized = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  /* eslint-disable no-param-reassign, no-underscore-dangle */
  if (memoized[id]) return memoized[id];

  var entity = entities.get(id);
  memoized[id] = _extends({}, entity, (0, _helpers.mapDeep)(entity._embedded || {}, function (embeddedId) {
    return denormalize(entities, embeddedId, memoized);
  }));

  return memoized[id];
};

var localEntities = exports.localEntities = function localEntities(state) {
  return state.wp.getIn(['entities']);
};

var selectQuery = exports.selectQuery = function selectQuery(name) {
  return (0, _reselect.createSelector)((0, _reselect.createSelector)(function (state) {
    return state.wp.getIn(['requestsByName', name]);
  }, function (state) {
    return state.wp.getIn(['requestsByQuery']);
  }, function (currentRequest, cache) {
    if (!currentRequest) return false;

    var cacheID = currentRequest.get('cacheID');
    var page = currentRequest.get('page');

    if (cacheID) {
      return currentRequest.merge(cache.getIn([cacheID, page])).merge(cache.getIn([cacheID, 'pagination']));
    }

    return currentRequest;
  }), localEntities, function (request, entities) {
    if (!request) {
      return {
        status: _requestStatus.pending,
        error: false,
        data: false
      };
    }

    if (!request.get('data')) {
      return _extends({
        data: false,
        error: false
      }, request.toJSON());
    }

    var memo = {};
    var data = request.get('data');
    if (data.toJSON) {
      data = data.toJSON();
    }

    return request.set('data', request.get('operation') === 'get' ? data.map(function (id) {
      return denormalize(entities, id, memo);
    }) : data).toJSON();
  });
};